function [IDENT]=sto_stab_mpd(sig,nblock,fs,n_ord)
% Extract modal shapes in complex conjugate form:
% IDENT matrix formed by Frequency of the poles (in Hz) Damping and mode shapes
%--------------------------------------------------------------------------
% sig                   Data matrix of measured outputs
% nblock:               number of block rows in Hankel matrices
%                       (nblock * #outputs) is the max. order that can be estimated
%                       Typically: nblock = 2 * (max order)/(#outputs)
% deltat                Sampling Time
% nn                    Array of model orders
%--------------------------------------------------------------------------
%	          Authors:   A.Saettone     January 2005
% Following Author(s):   G.Abbiati      August 2009
%                        L. Zanotti Fragonara  August 2009
%--------------------------------------------------------------------------
%Run identification for various model orders

deltat=1/fs;
ind=1;
IDENT=struct('ident',0,'order',0);

for jj=1:length(n_ord)
    n=n_ord(jj);  %Read order from stabilisation array
    
    
    [nr,nc]=size(sig);                              %Check signal dimensions
    
    if nr<nc
        sig=sig';
    end
    
    [~,nc]=size(sig);
    
    if jj==1
        [A,K,C,R,AUXin] = sto_pos(sig,nblock,n);
    else
        [A,K,C,R]=sto_pos(sig,nblock,n,AUXin);
    end
    
    [V,D]=eig(A);                                   %Compute eigenvalue problem from matrix A
    
    w=zeros(n,1);z=w;
    for ii=1:n
        w(ii)=abs(log(D(ii,ii)))/deltat;
        z(ii)=-real(log(D(ii,ii)))/abs(log(D(ii,ii)));
    end
    %Reorder the matrices:
    
    PHI=C*V;                                        %Modal shapes
    
    %Compute MPD of Complex Mode Shapes
%     
%     w0=vecnorm(PHI);
%     for ii=1:size(PHI,2)
%         
%         phi_i=[real(PHI(:,ii)),imag(PHI(:,ii))];
%         
%         [U2,S2,V1]=svd(phi_i);
%         
%         V1=V1';
%         V2=[V1(2,2) -V1(1,2)]';
%         
%         Theta=acos(abs((phi_i*V2)/(norm(phi_i,1)*norm(V2))));
%         MPD(ii)=sum(w0(ii).*Theta)/sum(w0);
%     end

    for ii=1:size(PHI,2)
        
        phi_i=[imag(PHI(:,ii)),real(PHI(:,ii))];
        
        [V1,S2]=eig(phi_i'*phi_i);
%         [U2,S2,V1]=svd(phi_i);
        [~,pos]=min(diag(S2));
        Vmin=V1(:,pos);
%         V1=V1';
%         V2=[V1(2,2) -V1(1,2)]';
        
        ThetaMP(ii)=atan(Vmin(2)/-Vmin(1));
        ImMP(:,ii)=norm(imag(exp(-1i*ThetaMP(ii))*PHI(:,ii)));
        ReMP(:,ii)=norm(real(exp(-1i*ThetaMP(ii))*PHI(:,ii)));
        MPD(ii)=ImMP(:,ii)./ReMP(:,ii);
    end
    
    mod=abs(PHI);                                   %Modal shape normalisation
    for ii=1:n
        mod(:,ii)=mod(:,ii)/max(mod(:,ii));
    end
    
    
    ph=angle(PHI);                                  %Phase are in radians
    
    for ii=1:1:nc
        ph1(ii,:)=ph(ii,:)-ph(1,:);
    end
    
    %     save PROVA
    %Assemble result matrix, convert from rad/s to Hz
    DUMMY=[w'/(2*pi);z';mod.*sign(cos(ph1))];
    IDENT(ind).ident=DUMMY(:,2:2:size(DUMMY,2));
    IDENT(ind).order=n;
    IDENT(ind).mpd=MPD(2:2:size(DUMMY,2));
    
    clear w z mod ph1 ph PHI V D A C DUMMY
    ind=ind+1;
    
    %     if exist('DUMMY','var')==0
    %         IDENT=[];
    %     else
    %         for ii=2:2:length(DUMMY(:,1))
    %             IDENT(ii/2,:)=DUMMY(ii,:);
    %         end
    %     end
    
end

